@@ -1,6 +1,6 @@
 package Spp;
 
-our $VERSION = '1.05';
+our $VERSION = '1.06';
 
 use Exporter;
 our @ISA       = qw(Exporter);
@@ -14,6 +14,7 @@ use Spp::Tools qw(read_file write_file trim to_json);
 use Spp::LintParser qw(lint_parser);
 use Spp::Match qw(match);
 use Spp::OptSppAst qw(opt_spp_ast);
+use Spp::IsAtom qw(is_match);
 
 sub get_ast_table {
    my $ast   = shift;
@@ -85,10 +86,16 @@ sub repl {
       exit() if $line eq "\n";
       $line = trim($line);
       my $match = match($parser, $line);
-      # say("match-> ", to_json($match));
-      my $ast = opt_spp_ast($match);
-      $ast = eval_spp_ast($ast);
-      say(".. ", to_json($ast));
+      if (is_match($match)) {
+         say("match-> ", to_json($match));
+         my $ast = opt_spp_ast($match);
+         $ast = eval_spp_ast($ast);
+         say(".. ", to_json($ast));
+      } else {
+         say "Syntax error! trace error..";
+         match($parser, $line, 1);
+         exit();
+      }
    }
 }
 
