@@ -2,24 +2,24 @@ package Spp::Match;
 
 use Exporter;
 our @ISA       = qw(Exporter);
-our @EXPORT_OK = qw(match report);
+our @EXPORT_OK = qw(match);
 
 use 5.012;
 no warnings "experimental";
 use Spp::Tools;
-use Spp::Cursor qw(new_cursor step getchar prechar);
+use Spp::Cursor qw(cursor go get_char pre_char cache recover);
 use Spp::IsChar;
-use Spp::IsAtom qw(is_false is_true is_bool is_atom is_sym is_str);
+use Spp::IsAtom qw(is_false is_match is_true is_bool is_atom is_sym is_str);
 use Spp::ToSpp qw(to_spp);
 
 sub match {
-   my ($parser, $text) = @_;
+   my ($parser, $text, $debug) = @_;
    my ($door, $table) = @{ $parser }; 
-   my $cursor = new_cursor($text, $table);
+   my $cursor = cursor($text, $table);
    my $door_rule = $cursor->{ns}{$door};
+   if (defined $debug) { $cursor->{debug} = 1 }
    my $match = match_rule($door_rule, $cursor);
-   report($cursor) if is_false($match);
-   return ['true'] if is_true($match);
+   return $match if is_bool($match);
    my $char = first($door);
    return [$door, $match] if is_upper($char);
    return $match if is_lower($char);
@@ -54,11 +54,50 @@ sub _match_rule {
    }
 }
 
+sub match_rule {
+   my ($x_rule, $cursor) = @_;
+   if ($cursor->{debug} == 1) {
+      my $name = $x_rule->[0];
+      # do not need trace rule
+      if ($name ~~ ['Group','Rept','Look',
+         'Branch','Lbranch','Ctoken']) {
+         return _match_rule($x_rule, $cursor);
+      }
+      my $off  = $cursor->{off};
+      my $str  = $cursor->{str};
+      my $rule = to_spp($x_rule);
+      my $flag = '->';
+      my $char = char_to_see(substr($str, $off, 1));
+      my $indent   = ' ' x $cursor->{depth};
+      my $log = "$off $char $indent $flag $rule";
+      $cursor->{depth}++;
+      my $match = _match_rule($x_rule, $cursor);
+      $cursor->{depth}--;
+      if (is_match($match)) { return $match }
+      say $log;
+      return $match;
+   }
+   else {
+      return _match_rule($x_rule, $cursor);
+   }
+}
+
+sub char_to_see {
+   my $char = shift;
+   given ($char) {
+     when ("\n") { return '\n' }
+     when ("\r") { return '\r' }
+     when ("\t") { return '\t' }
+     when (" ")  { return '\s' }
+     when (chr(0)) { return '\0' }
+     default { return " $char" }
+   }
+}
 sub match_any {
    my ($any, $cursor) = @_;
-   my $char = getchar($cursor);
+   my $char = get_char($cursor);
    return ['false'] if $char eq chr(0);
-   step($cursor);
+   go($cursor);
    return $char;
 }
 
@@ -70,17 +109,17 @@ sub match_assert {
          return ['false'];
       }
       when ('$') {
-         return ['true'] if getchar($cursor) eq chr(0);
+         return ['true'] if get_char($cursor) eq chr(0);
          return ['false'];
       }
       when ('^^') {
-         return ['true'] if prechar($cursor) eq "\n";
+         return ['true'] if pre_char($cursor) eq "\n";
          return ['true'] if $cursor->{off} == 0;
          return ['false'];
       }
       when ('$$') {
-         return ['true'] if getchar($cursor) eq "\n";
-         return ['true'] if getchar($cursor) eq chr(0);
+         return ['true'] if get_char($cursor) eq "\n";
+         return ['true'] if get_char($cursor) eq chr(0);
          return ['false'];
       }
       default { die("die assert char: |$assert|") }
@@ -100,11 +139,11 @@ sub match_rules {
 
 sub match_branch {
    my ($branch, $cursor) = @_;
-   my $cache = $cursor->{off};
+   my $cache = cache($cursor);
    for my $rule (@{$branch}) {
       my $match = match_rule($rule, $cursor);
       if (is_false($match)) {
-         $cursor->{off} = $cache;
+         recover($cursor, $cache);
       } else { return $match }
    }
    return ['false'];
@@ -112,34 +151,36 @@ sub match_branch {
 
 sub match_lbranch {
    my ($branch, $cursor) = @_;
-   my $cache      = $cursor->{off};
+   my $cache = cache($cursor);
    my $max_match  = ['false'];
-   my $max_length = $cache;
+   my $max_cache = $cache;
    for my $rule (@{$branch}) {
       my $match = match_rule($rule, $cursor);
-      if (!is_false($match)) {
-         if ($cursor->{off} >= $max_length) {
-            $max_length = $cursor->{off};
+      if (is_match($match)) {
+         if ($cursor->{off} >= $max_cache->[1]) {
+            $max_cache = cache($cursor);
             $max_match  = $match;
          }
       }
-      $cursor->{off} = $cache;
+      recover($cursor, $cache);
    }
-   $cursor->{off} = $max_length;
+   recover($cursor, $max_cache);
    return $max_match;
 }
 
 sub match_ntoken {
    my ($name, $cursor) = @_;
    my $rule  = $cursor->{ns}->{$name};
-   my $cache = $cursor->{off};
+   my $cache = cache($cursor);
+   my $from  = $cache->[1];
    my $match = match_rule($rule, $cursor);
    return $match if is_bool($match);
-   my $len    = $cursor->{off} - $cache;
-   my $str    = substr($cursor->{str}, $cache, $len);
+   my $len    = $cursor->{off} - $from;
+   my $str    = substr($cursor->{str}, $from, $len);
    my $c_name = '$' . $name;
    $cursor->{ns}->{$c_name} = ['Str', $str];
-   return name_match($name, $match);
+   push @{$cache}, $len;
+   return name_match($name, $match, $cache);
 }
 
 sub match_ctoken {
@@ -180,12 +221,12 @@ sub match_till {
    my @buf = ();
    # could not reach chr(0) how to match $
    while ($cursor->{off} < $cursor->{len}) {
-      my $char = getchar($cursor);
+      my $char = get_char($cursor);
       my $match = match_rule($rule, $cursor);
       # say "<$char>";
       if (is_false($match)) {
          push @buf, $char;
-         step($cursor);
+         go($cursor);
       } else {
          return gather_match(join('', @buf), $match);
       }
@@ -199,11 +240,11 @@ sub match_rept {
    my ($rept,   $atom)   = @{$rule};
    my ($min,    $max)    = get_rept_time($rept);
    while ($time != $max) {
-      my $cache = $cursor->{off};
+      my $cache = cache($cursor);
       my $match = match_rule($atom, $cursor);
       if (is_false($match)) {
          return ['false'] if $time < $min;
-         $cursor->{off} = $cache;
+         recover($cursor, $cache);
          return $gather;
       }
       $time++;
@@ -228,11 +269,11 @@ sub match_look {
    my ($atom,   $look)   = @atoms;
    my ($gather, $time)   = (['true'], 0);
    while ($time != $max) {
-      my $cache = $cursor->{off};
+      my $cache = cache($cursor);
       my $match = match_rule($atom, $cursor);
       if (is_false($match)) {
          return ['false'] if $time > $min;
-         $cursor->{off} = $cache;
+         recover($cursor, $cache);
          $match = match_rule($look, $cursor);
          return ['false'] if is_false($match);
          return gather_match($gather, $match);
@@ -240,9 +281,9 @@ sub match_look {
       $time++;
       $gather = gather_match($gather, $match);
       if ($time >= $min) {
-         $cache = $cursor->{off};
+         $cache = cache($cursor);
          $match = match_rule($look, $cursor);
-         if (is_false($match)) { $cursor->{off} = $cache }
+         if (is_false($match)) { recover($cursor, $cache) }
          else { return gather_match($gather, $match) }
       }
    }
@@ -251,30 +292,30 @@ sub match_look {
 
 sub match_str {
    my ($str, $cursor) = @_;
-   my $cache = $cursor->{off};
+   my $cache = cache($cursor);
    for my $char (split('', $str)) {
-      if ($char ne getchar($cursor)) {
-         $cursor->{off} = $cache;
+      if ($char ne get_char($cursor)) {
+         recover($cursor, $cache);
          return ['false'];
       }
-      step($cursor);
+      go($cursor);
    }
    return $str;
 }
 
 sub match_char {
    my ($char, $cursor) = @_;
-   return ['false'] if $char ne getchar($cursor);
-   step($cursor);
+   return ['false'] if $char ne get_char($cursor);
+   go($cursor);
    return $char;
 }
 
 sub match_chclass {
    my ($atoms, $cursor) = @_;
-   my $char = getchar($cursor);
+   my $char = get_char($cursor);
    for my $atom (@{$atoms}) {
       if (match_catom($atom, $char)) {
-         step($cursor);
+         go($cursor);
          return $char;
       }
    }
@@ -283,12 +324,12 @@ sub match_chclass {
 
 sub match_nchclass {
    my ($atoms, $cursor) = @_;
-   my $char = getchar($cursor);
+   my $char = get_char($cursor);
    return ['false'] if $char eq chr(0);
    for my $atom (@{$atoms}) {
       return ['false'] if match_catom($atom, $char);
    }
-   step($cursor);
+   go($cursor);
    return $char;
 }
 
@@ -305,10 +346,10 @@ sub match_catom {
 
 sub match_cclass {
    my ($cclass, $cursor) = @_;
-   my $char = getchar($cursor);
+   my $char = get_char($cursor);
    return ['false'] if $char eq chr(0);
    if (is_match_cclass($cclass, $char)) {
-      step($cursor);
+      go($cursor);
       return $char;
    }
    return ['false'];
@@ -380,10 +421,10 @@ sub match_array {
 }
 
 sub name_match {
-   my ($name, $match) = @_;
+   my ($name, $match, $pos) = @_;
    return $match if is_true($match);
-   return [$name, [$match]] if is_atom($match);
-   return [$name, $match];
+   return [$name, [$match], $pos] if is_atom($match);
+   return [$name, $match, $pos];
 }
 
 sub gather_match {
@@ -403,45 +444,6 @@ sub gather_match {
    return [ @{$gather}, @{$match} ];
 }
 
-sub match_rule {
-   my ($x_rule, $cursor) = @_;
-   if ($cursor->{debug} == 1) {
-      my $off      = $cursor->{off};
-      my $rule     = to_spp($x_rule);
-      my $flag     = '->';
-      my $pass_str = substr($cursor->{str}, $off, 10);
-      my $str      = to_trace_str($pass_str, 10);
-      my $indent   = ' ' x $cursor->{depth};
-      printf("%d|%s|%s %s %s\n", $off, $str, $indent, $flag, $rule);
-      $cursor->{depth}++;
-      my $match = _match_rule($x_rule, $cursor);
-      $cursor->{depth}--;
-      if   (is_false($match)) { $flag = '<-' }
-      else                      { $flag = 'ok' }
-      $off      = $cursor->{off};
-      $pass_str = substr($cursor->{str}, $off, 10);
-      $str      = to_trace_str($pass_str, 10);
-      printf("%d|%s|%s %s %s\n", $off, $str, $indent, $flag, $rule);
-      return $match;
-   }
-   else {
-      if ($cursor->{maxoff} < $cursor->{off}) {
-         $cursor->{maxoff} = $cursor->{off};
-      }
-      return _match_rule($x_rule, $cursor);
-   }
-}
-
-sub report {
-   my $cursor   = shift;
-   my $off      = $cursor->{maxoff};
-   my $pass_str = substr($cursor->{str}, $off, 10);
-   my $line     = count($pass_str, "\n") + 1;
-   my $str      = to_trace_str($pass_str, 20);
-   printf("block at: line: %d |^%s\n", $line, $str);
-   exit();
-}
-
 sub eval_atom {
    my ($atom, $ns) = @_;
    if (!is_atom($atom)) {
