@@ -1,12 +1,8 @@
-# Copyright 2016 The Michael Song. All rights rberved.
-# Use of this source code is governed by a BSD-style
-# license that can be found in the LICENSE file.
-
 package Spp::ToSpp;
 
 use Exporter;
-our @ISA       = qw(Exporter);
-our @EXPORT_OK = qw(to_spp);
+our @ISA    = qw(Exporter);
+our @EXPORT = qw(to_spp);
 
 use 5.012;
 no warnings "experimental";
