@@ -5,7 +5,7 @@ our @ISA    = qw(Exporter);
 our @EXPORT = qw(all is_perl_str is_perl_array
    read_file write_file str to_json from_json
    count start_with end_with 
-   first tail rest trim len to_trace_str subarray);
+   first tail rest trim len subarray);
 
 use 5.012;
 no warnings "experimental";
@@ -92,7 +92,8 @@ sub start_with {
 
 sub end_with {
    my ($str, $end) = @_;
-   return start_with(reverse_str($str), reverse_str($end));
+   my $len = length($end);
+   return substr($str, -$len) eq $end;
 }
 
 sub len {
@@ -110,34 +111,6 @@ sub trim {
    die("trim only make string");
 }
 
-sub to_trace_str {
-   my ($str, $n) = @_;
-   $str = reverse_str($str);
-   my @buf = ();
-   for my $ch (split('', $str)) {
-      given ($ch) {
-         when ("\n") { push @buf, 'n\\' }
-         when ("\r") { push @buf, 'r\\' }
-         when ("\t") { push @buf, 't\\' }
-         default     { push @buf, $ch }
-      }
-   }
-   $str = join('', @buf);
-   my $len = len($str);
-   if ($len > $n) {
-      $str = substr($str, 0, $n);
-   }
-   elsif ($len < $n) {
-      $str = (' ' x ($n - $len)) . $str;
-   }
-   return reverse_str($str);
-}
-
-sub reverse_str {
-   my $str = shift;
-   return scalar(reverse($str));
-}
-
 sub subarray {
   my ($array, $from, $to) = @_;
   # make copy of $array, splice would change it
